# -*- encoding:utf-8 -*-
import logging
import os
from logging import handlers
import json
import datetime
import socket

LOG_PATH = os.environ.get('LOG_PATH') if os.environ.get('LOG_PATH') else '/tmp'
REMOVE_ATTR = ["filename", "module", "exc_text", "stack_info", "created", "msecs", "relativeCreated", "exc_info", "msg",
               "args"]


class HostIp:
    host_name = None
    host_ip = None

    @classmethod
    def get_host_ip(cls):
        if not cls.host_name or not HostIp.host_ip:
            try:
                cls.host_name = socket.gethostname()
                cls.host_ip = socket.gethostbyname(cls.host_name)
            except:
                cls.host_name = "unknown hostname"
                cls.host_ip = "unknown ip"
        return cls.host_name, cls.host_ip


class JSONFormatter(logging.Formatter):
    host_name, host_ip = HostIp.get_host_ip()

    def format(self, record):
        extra = self.build_record(record)
        self.set_format_time(extra)  # set time
        self.set_host_ip(extra)  # set host name and host ip
        if isinstance(record.msg, dict):
            extra['data'] = record.msg  # set message
        else:
            if record.args:
                extra['msg'] = "'" + record.msg + "'," + str(record.args).strip('()')
            else:
                extra['msg'] = record.msg
        if record.exc_info:
            extra['exc_info'] = self.formatException(record.exc_info)
        if self._fmt == 'pretty':
            return json.dumps(extra, indent=1, ensure_ascii=False)
        else:
            return json.dumps(extra, ensure_ascii=False)

    @classmethod
    def build_record(cls, record):
        return {
            attr_name: record.__dict__[attr_name]
            for attr_name in record.__dict__
            if attr_name not in REMOVE_ATTR
        }

    @classmethod
    def set_format_time(cls, extra):
        now = datetime.datetime.utcnow()
        format_time = now.strftime("%Y-%m-%dT%H:%M:%S" + ".%03d" % (now.microsecond / 1000) + "Z")
        extra['@timestamp'] = format_time
        return format_time

    @classmethod
    def set_host_ip(cls, extra):
        extra['host_name'] = JSONFormatter.host_name
        extra['host_ip'] = JSONFormatter.host_ip


class Logger(object):
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'crit': logging.CRITICAL
    }  # 日志级别关系映射

    def __init__(self, name=None, level='info', when='D', backup_count=7,
                 fmt='%(asctime)s - %(pathname)s[line:%(lineno)d] - %(levelname)s: %(message)s'):
        self.logger = logging.getLogger(name)
        format_str = logging.Formatter(fmt)  # 设置日志格式
        self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
        sh = logging.StreamHandler()  # 往屏幕上输出
        sh.setFormatter(format_str)  # 设置屏幕上显示的格式
        self.logger.addHandler(sh)  # 把对象加到logger里
        if name:
            th = handlers.TimedRotatingFileHandler(filename=os.path.join(LOG_PATH, f'{name}.log'),
                                                   when=when,
                                                   backupCount=backup_count,
                                                   encoding='utf-8')  # 往文件里写入#指定间隔时间自动生成文件的处理器
            # 实例化TimedRotatingFileHandler
            # interval是时间间隔，backupCount是备份文件的个数，如果超过这个个数，就会自动删除，when是间隔的时间单位，单位有以下几种：
            # S 秒
            # M 分
            # H 小时、
            # D 天、
            # W 每星期（interval==0时代表星期一）
            # midnight 每天凌晨
            th.setFormatter(JSONFormatter())  # 设置文件里写入的格式
            self.logger.addHandler(th)



# -*-coding:utf-8 -*-

"""
# File       : s3_file.py
# Time       ：2021/1/22 10:21 上午
# Author     ：shen
# version    ：python 3.7
# Description：
"""
import boto3
import re
import gzip
import os


def download_s3(filter_str,
                bucket,
                access_key_id,
                secret_access_key,
                local_root_path):
    s3 = boto3.resource(service_name='s3',
                        aws_access_key_id=access_key_id,
                        aws_secret_access_key=secret_access_key,
                        region_name='cn-north-1')
    bucket = s3.Bucket(bucket)

    for bucket_object in bucket.objects.all():
        key = bucket_object.key
        if key[:len(filter_str)] == filter_str and 'xml.gz' in key:
            local_path = os.path.join(local_root_path, key)
            if os.path.exists(local_path):
                continue
            try:
                print('downloading ', key)
                bucket.download_file(key, local_path)
            except:
                path = '/'.join(key.split('/')[:-1])
                path_mk = os.path.join(local_root_path, path)
                print('mkdir ', path_mk)
                os.makedirs(path_mk)
                bucket.download_file(key, local_path)
            print(f'downloaded to {local_path}')


def s3_list(filter_str, bucket, aws_access_key_id, aws_secret_access_key):
    s3 = boto3.resource(service_name='s3',
                        aws_access_key_id=aws_access_key_id,
                        aws_secret_access_key=aws_secret_access_key,
                        region_name='cn-north-1')
    bucket = s3.Bucket(bucket)
    print(f'{filter_str}:')
    print(f'====================')
    for bucket_object in bucket.objects.all():
        key = bucket_object.key
        if key[:len(filter_str)] == filter_str:
            print(key)
    print(f'====================')


def un_gz(file_name):
    f_name = file_name.replace(".gz", "")
    if os.path.exists(f_name):
        return
    g_file = gzip.GzipFile(file_name)
    open(f_name, "wb+").write(g_file.read())
    return f_name


def run_unzip_file(file_path):
    # 解压缩gz文件 -> xml
    f1 = []
    r1 = []
    get_file_path(file_path, f1, r1, file_style='.*gz$')
    for r in f1:
        file_name = un_gz(r)
        if file_name:
            print(f'saved {file_name}')


def get_file_path(root_path, file_list, dir_list, file_style='.csv', dir_style=''):
    dir_or_files = os.listdir(root_path)
    dir_or_files = list(filter(lambda x: x[0] != '.', dir_or_files))
    for dir_file in dir_or_files:
        dir_file_path = os.path.join(root_path, dir_file)
        if os.path.isdir(dir_file_path):
            if dir_style in dir_file_path:
                dir_list.append(dir_file_path)
                get_file_path(dir_file_path, file_list, dir_list, file_style=file_style)
        else:
            if re.search(re.compile(file_style), dir_file_path):
                # if file_style in dir_file_path:
                file_list.append(dir_file_path)



# -*- encoding:utf-8 -*-
import pandas as pd
import json
import redis
import datetime
from dateutil import tz

UTC = tz.gettz('UTC')
BJ = tz.gettz('Asia/Shanghai')
ET = tz.gettz('America/New_York')


class RedisObject(object):
    def __init__(self, host, port):
        pool = redis.ConnectionPool(host=host, port=port)
        self.rds = redis.Redis(connection_pool=pool)

    def hash_to_dataframe_new(self, key, redis_timestamp_tz=None, local_tz=None,
                              columns=None, start_time=None, end_time=None, is_trans_time=True):

        if not columns:
            columns = ['tradeTime', 'open', 'close', 'high', 'low', 'volume']

        res_dict = self.rds.hgetall(key)
        # columns.remove('tradeTime')
        if 'tradeTime' in columns:
            datetime_list = json.loads(res_dict[b'tradeTime'])
            if is_trans_time:
                if not redis_timestamp_tz:
                    datetime_list = list(
                        map(lambda x: datetime.datetime.utcfromtimestamp(x),
                            datetime_list))
                else:
                    datetime_list = list(
                        map(lambda x: datetime.datetime.fromtimestamp(x, tz=redis_timestamp_tz).astimezone(local_tz),
                            datetime_list))
        else:
            raise Exception('`tradeTime` not in columns')

        pd_dict = dict()
        for column in columns:
            if column == 'tradeTime':
                pd_dict.update({column: datetime_list})
            else:
                pd_dict.update({column: json.loads(res_dict[bytes(column, encoding='utf-8')])})
        if 'tradeTime' in columns:
            df = pd.DataFrame(pd_dict, index=datetime_list)
        else:
            df = pd.DataFrame(pd_dict)

        if not redis_timestamp_tz:
            if start_time:
                start_time = start_time.replace(tzinfo=None)
            if end_time:
                end_time = end_time.replace(tzinfo=None)
        else:
            if start_time:
                start_time = start_time.replace(tzinfo=local_tz)
            if end_time:
                end_time = end_time.replace(tzinfo=local_tz)
        if not start_time and not end_time:
            pass
        elif start_time and end_time:
            df = df.loc[(df.index >= start_time) & (df.index <= end_time)]
        elif start_time and not end_time:
            df = df.loc[df.index >= start_time]
        else:
            df = df.loc[df.index <= end_time]
        return df

    def get_dataframe_from_redis(self, key, fromdate=datetime.datetime(1970, 2, 1, tzinfo=ET),
                                 todate=datetime.datetime.now().astimezone(tz=ET), ktype='d1', columns=None,
                                 redis_timestamp_tz=None,
                                 local_tz=ET
                                 ):
        """
        根据tickerId获取指定时间段内、指定列的df
        :param key: str
        :param fromdate: datetime
        :param todate: datetime
        :param ktype: str
        :param columns: list, list of str
        :param redis_timestamp_tz: dateutil.tz
        :param local_tz: dateutil.tz
        :return: df
        """
        f_stamp = fromdate.timestamp()
        t_stamp = todate.timestamp()
        pattern = f'*{ktype}*'

        if ':' not in key:
            match_list = self.rds.keys(pattern=pattern + f'{key}*')
            match_list = list(map(lambda i: str(i, encoding="utf-8"), match_list))
            filter_list = []
            for m in match_list:
                ml = m.split(':')
                stime, etime = int(float(ml[-3])), int(float(ml[-2]))
                if stime <= t_stamp and f_stamp <= etime:
                    filter_list.append(m)
            dataframe = []
            for key in filter_list:
                dataframe.append(self.hash_to_dataframe_new(key,
                                                            redis_timestamp_tz=redis_timestamp_tz,
                                                            local_tz=local_tz,
                                                            start_time=fromdate,
                                                            end_time=todate, columns=columns))
            if dataframe:
                copy_dataframe = dataframe.copy()
                for d, i in zip(copy_dataframe, range(len(copy_dataframe))):
                    if d.empty:
                        del dataframe[i]
            if dataframe:
                dataname = pd.concat(dataframe)
                dataname = dataname.sort_values(by="tradeTime", ascending=True)
            else:
                dataname = pd.DataFrame()

        else:
            dataname = self.hash_to_dataframe_new(key,
                                                  redis_timestamp_tz=redis_timestamp_tz,
                                                  local_tz=local_tz,
                                                  )
        return dataname



# -*-coding:utf-8 -*-

"""
1. python3 setup.py sdist bdist_wheel
2. twine upload dist/*
"""
from __future__ import print_function
from setuptools import setup, find_packages
import sys

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="datacenter_utils",
    version="0.2.5",
    author="Wang Sheng",
    author_email="jayed008@163.com",
    description="datacenter_utils",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Environment :: Web Environment",
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    install_requires=[
        'pandas>=0.25.3',
        'numpy>=1.18.4',
        'redis>=3.5.2',
        'pymongo',
        'boto3',
        'python-dateutil',
        'psycopg2-binary~=2.8.6',
        'pyyaml'
    ],
    zip_safe=True,
)



# -*-coding:utf-8 -*-

import psycopg2
from psycopg2 import OperationalError
from datetime import datetime, date
import pandas as pd
import re
import numpy as np
import yaml
import os
import json


def read_yaml(file_path, key='pg'):
    with open(file_path, 'r') as f:
        cfg = yaml.safe_load(f)
    return cfg[key]


class Postgres(object):

    def __init__(self, conf=None, conf_path='/etc/db.yaml'):
        """
        生成数据库连接self.cursor。默认加载conf配置，当conf配置不存在时，再加载环境变量PG_CONF的配置，
        若PG_CONF的配置也不存在，最后加载conf_path（默认/etc/db.yaml）的文件配置
        优先级：conf字典配置 > 环境变量配置 > conf_path文件配置

        :param conf: 对应数据库配置
        :param conf_path: yaml配置文件绝对路径（可放在环境下任意路径），文件格式参考同目录下的db.yaml
        """
        if not conf:
            conf_str = os.environ.get('PG_CONF')
            if conf_str:
                conf = eval(conf_str)
                if isinstance(conf, str):
                    conf = eval(conf)
            if not conf:
                conf = read_yaml(conf_path)
        try:
            self.database_postgres = psycopg2.connect(database=conf['database'],
                                                      user=conf['user'],
                                                      password=conf['password'],
                                                      host=conf['host'],
                                                      port=int(conf['port']))

            self.cursor = self.database_postgres.cursor()
        except OperationalError as e:
            raise e

    def is_table_exist(self, table_str):
        # TODO 表名字符长度限制63
        exec_str = f"select exists(select * from information_schema.tables where table_name='{table_str.lower()[:63]}');"
        self.cursor.execute(exec_str)
        exist = self.cursor.fetchone()[0]
        print(f'{table_str} is_table_exist: {exist}')
        return exist

    def update_insert_df(self, df, table_name, text_columns, constraint_columns,
                         date_columns=None, timestamp_columns=None
                         ):
        """
        dataframe按照规定的字段设计upsert到指定数据库。
        针对constraint_columns列表，df任一记录任一字段与数据库不一样，就会插入一条新的数据，否则会更新数据库里原有记录。
        当表（table_name）不存在时，该函数操作会自动创建表，按照字段类型参数（text_columns、constraint_columns、date_columns、
        timestamp_columns）创建相应类型，未指定的按照text类型处理

        :param df: 待插入到数据库里的dataframe，其columns需要明确且与text_columns、constraint_columns、date_columns、timestamp_columns保持一致
        :type df: pandas.Dataframe
        :param table_name: 目标表名
        :type table_name: str
        :param text_columns: 定义为text类型的字段
        :type text_columns: list of str
        :param constraint_columns: 设置为CONSTRAINT
        :type constraint_columns: list of str
        :param date_columns: 定义为date类型的字段
        :type date_columns: list of str
        :param timestamp_columns:  定义为timestamp类型的字段
        :type timestamp_columns: list of str
        """
        data_list = df.values.tolist()
        columns = list(df.columns)
        data_list.insert(0, columns)
        if date_columns:
            text_columns.extend(date_columns)
        if timestamp_columns:
            text_columns.extend(timestamp_columns)
        # self.cursor.execute(f'DROP TABLE IF EXISTS "{table_name}";')
        count = 0
        column_list = []
        c_str = ''
        for row in data_list:
            if count == 0:
                # 标题format
                column_list = [self._format_str(r) for r in row]

                if not text_columns:
                    text_columns = column_list
                sql_str = ','.join([f'"{j.replace(" ", "_")}" float' if j not in text_columns
                                    else f'"{j.replace(" ", "_")}" text'
                                    for j in column_list])
                if not date_columns:
                    date_columns = []
                for dd in date_columns:
                    sql_str = sql_str.replace(f'"{dd}" text', f'"{dd}" date')

                if not timestamp_columns:
                    timestamp_columns = []
                for dd in timestamp_columns:
                    sql_str = sql_str.replace(f'"{dd}" text', f'"{dd}" timestamp')

                # sql_s = re.findall(r'[^\*/:?\\|<>!]', sql_s, re.S)
                # sql_s = "".join(sql_s)
                print(sql_str)
                # 自定义数据类型
                if not self.is_table_exist(table_name):
                    add_conflict_sql = '''
                    ALTER TABLE "public"."{t}" ADD CONSTRAINT "{pkey}" PRIMARY KEY ({keys});
                    '''.format(t=table_name, pkey=table_name + '_pkey',
                               keys=','.join([f'"{i}"' for i in constraint_columns])
                               )
                    self.cursor.execute(f'create table {table_name} ({sql_str});')
                    self.cursor.execute(add_conflict_sql)
                    print('create table', sql_str)
            else:
                if len(column_list) - len(constraint_columns) == 1:
                    sql = '''
                               INSERT INTO {t} 
                                    VALUES  ({column_str})
                                    ON CONFLICT ({constraint_columns})
                                    DO UPDATE SET
                                        {exclude_columns}
                                        = {exclude_values} ;
                            '''
                elif len(column_list) - len(constraint_columns) > 1:
                    sql = '''
                               INSERT INTO {t} 
                                    VALUES  ({column_str})
                                    ON CONFLICT ({constraint_columns})
                                    DO UPDATE SET
                                        ({exclude_columns})
                                        = ({exclude_values}) ;
                            '''
                else:
                    sql = ''''''

                for i in range(len(row)):
                    if date_columns and column_list[i] in date_columns:
                        if isinstance(row[i], pd.Timestamp):
                            row[i] = row[i].date()
                        elif isinstance(row[i], str):
                            row[i] = datetime.strptime(row[i], '%Y-%m-%d').date()
                        elif isinstance(row[i], date):
                            pass
                        elif isinstance(row[i], datetime):
                            row[i] = row[i].date()
                        else:
                            row[i] = None

                        if timestamp_columns and column_list[i] in timestamp_columns:
                            if isinstance(row[i], pd.Timestamp):
                                row[i] = row[i].datetime()
                            elif isinstance(row[i], str):
                                row[i] = datetime.strptime(row[i], '%Y-%m-%d %H:%M:%S')
                            elif isinstance(row[i], date):
                                pass
                            elif isinstance(row[i], datetime):
                                pass
                            else:
                                row[i] = None
                    # print(type(row[i]),row[i])
                    if row[i] == '' or (isinstance(row[i], float) and np.isnan(row[i])):
                        row[i] = None
                    # if filter_dict and column_list[i] in filter_dict.keys():
                    #     if filter_dict[column_list[i]] == 'text_format':
                    #         row_s = str(row[i])
                    #         row_s = re.findall(r'[^\*/:?\\|<>!\-]', row_s, re.S)
                    #         row[i] = "".join(row_s)

                # one_sql_str = ','.join(row)
                if not c_str:
                    c_str = ','.join(['%s' for i in range(len(row))])
                    exclude_list = column_list.copy()
                    for remove_c in constraint_columns:
                        exclude_list.remove(remove_c)
                    exclude_columns = [f'"{i}"' for i in exclude_list]
                    constraint_columns = [f'"{i}"' for i in constraint_columns]
                    exclude_values = [f'EXCLUDED."{i}"' for i in exclude_list]
                    sql_str = sql_str.replace('text', '').replace('date', '').replace('float', '')
                sql_format = sql.format(t=table_name,
                                        column_str=c_str,
                                        constraint_columns=','.join(constraint_columns),
                                        exclude_columns=','.join(exclude_columns),
                                        exclude_values=','.join(exclude_values))
                # print(row)
                # print(sql_format)
                self.cursor.execute(sql_format, row)
            count += 1

        print(f'saved {table_name}')
        self.database_postgres.commit()

    def find(self, table, filter_dict, columns):
        """
        查询所有满足条件的结果

        :param table: 表格名称
        :param filter_dict: 筛选字典
        :param columns: 返回字段
        :type table: str
        :type filter_dict: dict
        :type columns: list of str
        :return: 查询结果
        """
        columns_str = ",".join([f'"{i}"' for i in columns])
        filter_list = []
        for k, v in filter_dict.items():
            if isinstance(v, str):
                v = f"'{v}'"
            elif isinstance(v, (datetime, date)):
                v = f"'{v}'"
            filter_list.append(f'"{k}" = {v}')
        filter_str = ' and '.join(filter_list)
        sql = f"select {columns_str} from {table} where {filter_str}"
        print('sql:', sql)
        self.cursor.execute(sql)
        res = self.cursor.fetchall()
        return res

    def find_one(self, table, filter_dict, columns):
        """
        查询满足条件的一个结果

        :param table: 表格名称
        :param filter_dict: 筛选字典
        :param columns: 返回字段
        :type table: str
        :type filter_dict: dict
        :type columns: list of str
        :return: 查询结果
        """
        columns_str = ",".join([f"{i}" for i in columns])
        filter_list = []
        for k, v in filter_dict.items():
            if isinstance(v, str):
                v = f"'{v}'"
            elif isinstance(v, (datetime, date)):
                v = f"'{v}'"
            filter_list.append(f'"{k}" = {v}')
        filter_str = ' and '.join(filter_list)
        sql = f"select {columns_str} from {table} where {filter_str}"
        print('sql:', sql)
        self.cursor.execute(sql)
        res = self.cursor.fetchone()
        return res

    def find_by_sql(self, sql):
        """
        通过sql语句查询，返回所有结果
        :param sql: 查询语句
        :return:
        """
        self.cursor.execute(sql)
        res = self.cursor.fetchall()
        return res

    def close_connect(self):
        """
        关闭数据库连接，查询、更新操作结束需要执行此操作
        """
        self.database_postgres.close()

    @staticmethod
    def _format_str(line):
        # 指定字符替换为 -
        line1 = re.sub(r"[\-\n\s \t]", "_", line)
        # 指定字符去除
        line2 = re.sub(r"[‘’“”，。？?]", "", line1)
        return line2



# -*- encoding:utf-8 -*-
import pandas as pd
from pymongo import MongoClient, ASCENDING, DESCENDING
import time
from datetime import datetime, timedelta
from pymongo.errors import OperationFailure


class MongoObject(object):
    """
    class MongoObject
    """
    def __init__(self, host, port, username, password, auth_db='admin'):
        self.insert_bench = 6000
        self.client = MongoClient(f"mongodb://{host}:{port}")
        db = self.client[auth_db]
        db.authenticate(username, password)

    def upsert_many_df(self, collection, df_data, key_field, update_field):
        """
        dataframe按照规定的字段设计upsert到指定数据库

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param df_data: dataframe，需要插入mongodb的df数据
        :param key_field: list，确定唯一性的一组字段，任一字段变化都新增一条数据，否则则更新其他字段
        :param update_field: list，上述需要更新的字段列表
        """
        df_data_js = df_data.to_dict(orient='records')  # json.loads(df_data.T.to_json())
        # 添加需要的索引，即主键
        for index_field in key_field:
            collection.create_index(index_field)
        # 写入操作
        bulk = collection.initialize_unordered_bulk_op()

        for num in range(len(df_data_js)):
            item = df_data_js[num]
            insert_data = self._join_upsert_dict(item, key_field, update_field)

            bulk.find(insert_data['key_data']).upsert().update_one(insert_data['change_data'])

            if num % self.insert_bench == 0:
                bulk.execute()
                bulk = collection.initialize_unordered_bulk_op()
            if num == len(df_data_js) - 1:
                bulk.execute()

    @staticmethod
    def _join_upsert_dict(item, key_field, update_field):
        # key_data_dict: if key data exist remain else insert new
        key_data_dict = {}
        for field in key_field:
            joint_dict = {field: item[field]}
            key_data_dict = {**key_data_dict, **joint_dict}

        # change_data_dict ,data need to be update
        change_data_dict = {}
        for field in update_field:
            joint_dict = {field: item[field]}
            change_data_dict = {**change_data_dict, **joint_dict}
        joint_dict = {"updateTime": datetime.utcfromtimestamp(time.time())}
        change_data_dict = {**change_data_dict, **joint_dict}
        change_data_dict = {'$set': change_data_dict}

        # join key_data and change_data
        insert_data = {"key_data": key_data_dict, "change_data": change_data_dict}
        return insert_data

    @staticmethod
    def remove(collection, filter_dict):
        """
        删除满足条件的数据

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param filter_dict: dict，需要删除数据的筛选条件
        """
        try:
            collection.remove(filter_dict)
        except:
            collection.delete_many(filter_dict)

    @staticmethod
    def insert_one(collection, data):
        """
        插入一条数据字典

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param data: dict，新增的数据字典
        """
        res = collection.insert_one(data)
        return res.inserted_id

    @staticmethod
    def insert_many(collection, data):
        """
        插入数据列表

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param data: list，新增的数据列表
        """
        res = collection.insert_many(data)
        return res.inserted_ids

    @staticmethod
    def update_one(collection, condition, data):
        """
        更新一条记录

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param condition: dict，被更新数据满足的条件
        :param data: dict，需要更新到的状态
        """
        res = collection.find_one(condition)
        if not res:
            data.update(condition)
            insert_res = collection.insert_one(data)
            return 0, insert_res
        else:
            update_result = collection.update(condition, {'$set': data})
            return 1, update_result

    @staticmethod
    def find_many_df(collection, filter_dict=None, projection=None, order_by=None,
                     offset=0, limit=20
                     ):
        """
        从指定表获取满足筛选条件的数据，返回dataframe

        :param collection: collection connect object，通过实例化MongoObject的mongo_object[db_name][table_name]得到
        :param filter_dict: dict，筛选字段
        :param projection: dict，规定返回字段和不返回的字段，{<c1>: 1, <c2>: -1}表示字段c1返回、字段c2不返回
        :param order_by: str，规定排序的字段，'-c1'表示对字段c1逆序排列，'-c2'表示对字段c2顺序排列
        :param offset: int，截取offset值以后的记录
        :param limit: int，截取offset值以后的limit条记录
        :return:
        """
        cursor = None
        if filter_dict and not projection:
            projection = dict()
            projection.update({'_id': 0})
            cursor = collection.find(filter_dict).limit(limit).skip(offset)
        if filter_dict and projection:
            projection.update({'_id': 0})
            cursor = collection.find(filter_dict, projection).limit(limit).skip(offset)
        if not filter_dict and not projection:
            cursor = collection.find({}).limit(limit).skip(offset)

        if order_by:
            order = -1 if '-' in order_by else 1
            order_by = order_by.replace('-', '')
            cursor.sort([(order_by, order)])

        if cursor:
            docs = list(cursor)
            return pd.DataFrame(docs)
